package org.apache.hadoop.dfs;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;

import org.apache.hadoop.io.Writable;

/**
 * NetFlowTable has a minimal interface for maintaining NetFlow's.
 * It is synchronized because the usage is very clear, and only in TaskTracker.
 * 
 * @author sko
 *
 */
public class NetFlowTable implements Writable {

  private HashMap<String, NetFlow> flowTable;
  
  public NetFlowTable() {
    this.flowTable = new HashMap<String, NetFlow>();
  }
  
  public NetFlow updateFlow(NetFlow f) {
    NetFlow ret = null;

    // only update what's been added explicitly
    synchronized (flowTable) {
      if (flowTable.containsKey(f.getMapTaskId() +
          " > " + f.getReduceTaskId()))
        ret = flowTable.put(f.getMapTaskId() + " > " + f.getReduceTaskId(), f);
    }

    return ret; 
  }
  
  public NetFlow put(NetFlow f) {
    NetFlow ret = null;
    
    if (f.getMapTaskId() != null && f.getReduceTaskId() != null) {
      synchronized (flowTable) {
        ret = flowTable.put(f.getMapTaskId() + " > " + f.getReduceTaskId(), f);
      }
    }
    
    return ret;
  }
  
  public NetFlow remove(NetFlow f) {
    NetFlow ret = null;
    if (f.getMapTaskId() != null && f.getReduceTaskId() != null) {
      synchronized (flowTable) {
        ret = flowTable.remove(f.getMapTaskId() + " > " + f.getReduceTaskId());
      }
    }
    
    return ret;
  }
  
  public void readFields(DataInput in) throws IOException {
    int c = in.readInt();
    if (c != 0) {
      for (int i = 0; i < c; ++i) {
        NetFlow f = new NetFlow(null, null, null, null, 0, 0);
        f.readFields(in);
        flowTable.put(f.getMapTaskId() + " > " + f.getReduceTaskId(), f);
      }
    } else
      flowTable = null;
  }

  public void write(DataOutput out) throws IOException {
    int c = 0;
    if (flowTable != null) {
      Collection<NetFlow> col = null;
      synchronized (flowTable) {
        col = flowTable.values();
      }
      
      for (NetFlow f: col)
        if (f.getMapTaskId() != null && f.getReduceTaskId() != null)
          c++;
      out.writeInt(c);
      for (NetFlow f: col)
        if (f.getMapTaskId() != null && f.getReduceTaskId() != null) {
          synchronized (f) {
            f.write(out);
          }
        }
    } else
      out.writeInt(0);
  }

  /*
  public String toString() {
    String ret = null;
    Collection<NetFlow> col = null;
    synchronized (flowTable) {
      col = flowTable.values();
    }

    for (NetFlow f: col) {
      synchronized (f) {
        ret += f + "\n";
      }
    }
    
    return ret;
  }
  */
}
